#/*##########################################################################
#
# The fisx library for X-Ray Fluorescence
#
# Copyright (c) 2018 European Synchrotron Radiation Facility
#
# This file is part of the fisx X-ray developed by V.A. Sole
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
__author__ = "V.A. Sole - ESRF Data Analysis"
import unittest
import sys
import os

class testDetector(unittest.TestCase):
    def setUp(self):
        """
        import the module
        """
        try:
            from fisx import Detector
            self._module = Detector
        except:
            self._module = None

    def tearDown(self):
        self._module = None

    def testDetectorImport(self):
        self.assertTrue(self._module is not None,
                        'Unsuccessful fisx.Detector import')

    def testDetectorInstantiation(self):
        try:
            instance = self._module("Dummy")
            instance = self._module(b"Dummy")
        except:
            instance = None
            print("Instantiation error: ",
                    sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
        self.assertTrue(instance is not None,
                        'Unsuccesful Detector() instantiation')

    def testDetectorResults(self):
        from fisx import Elements
        from fisx import Detector

        elementsInstance = Elements()
        elementsInstance.initializeAsPyMca()
        
        # Make a detector from a formula
        detectorName = "NaI"
        detectorDensity = 4.0
        detectorThickness = 0.00350
        detectorInstance = Detector(detectorName,
                                    detectorDensity,
                                    detectorThickness)

        # Check the composition is returned as string and not bytes under
        # Python 3
        composition = detectorInstance.getComposition(elementsInstance)
        if sys.version_info >= (3, ):
            for key in composition:
                self.assertTrue(isinstance(key, str),
                                "Expected string, received %s" % type(key))

        # check the returned keys are correct
        self.assertTrue(len(list(composition.keys())) == 2,
                        "Incorrect number of keys returned")
        for key in ["Na", "I"]:
            self.assertTrue(key in composition,
                            "key %s not found" % key)

        thickness = detectorInstance.getThickness()
        self.assertTrue(abs( thickness - detectorThickness) < 1.0e-7,
                       "Wrong detector thickness!")

        density = detectorInstance.getDensity()
        self.assertTrue(abs(density - detectorDensity) < 1.0e-7,
                       "Wrong detector density!")

        # check the return of the getEscape method
        escape = detectorInstance.getEscape(100.0, elementsInstance)
        if sys.version_info >= (3, ):
            for key in escape:
                self.assertTrue(isinstance(key, str),
                                "Expected string, received %s" % type(key))

def getSuite(auto=True):
    testSuite = unittest.TestSuite()
    if auto:
        testSuite.addTest(\
            unittest.TestLoader().loadTestsFromTestCase(testDetector))
    else:
        # use a predefined order
        testSuite.addTest(testDetector("testDetectorImport"))
        testSuite.addTest(testDetector("testDetectorInstantiation"))
        testSuite.addTest(testDetector("testDetectorResults"))
    return testSuite

def test(auto=False):
    unittest.TextTestRunner(verbosity=2).run(getSuite(auto=auto))

if __name__ == '__main__':
    test()



#/*##########################################################################
#
# The fisx library for X-Ray Fluorescence
#
# Copyright (c) 2014-2016 European Synchrotron Radiation Facility
#
# This file is part of the fisx X-ray developed by V.A. Sole
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
__author__ = "V.A. Sole - ESRF Data Analysis"
import unittest
import sys
import os
import tempfile

import numpy
import time
import os
import sys

try:
    from PyMca5.PyMca import PyMcaEPDL97
    PYMCA = True
except ImportError:
    # do not compare with PyMca because it is not installed
    PYMCA = False

ElementList= ['H', 'He', 
            'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne',
            'Na', 'Mg', 'Al', 'Si', 'P', 'S', 'Cl', 'Ar',
            'K', 'Ca', 'Sc', 'Ti', 'V', 'Cr', 'Mn', 'Fe',
            'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge', 'As', 'Se',
            'Br', 'Kr', 'Rb', 'Sr', 'Y', 'Zr', 'Nb', 'Mo',
            'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn',
            'Sb', 'Te', 'I', 'Xe', 'Cs', 'Ba', 'La', 'Ce',
            'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 
            'Ho', 'Er', 'Tm', 'Yb', 'Lu', 'Hf', 'Ta', 'W', 
            'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 
            'Bi', 'Po', 'At', 'Rn', 'Fr', 'Ra', 'Ac', 'Th', 
            'Pa', 'U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 
            'Es', 'Fm', 'Md', 'No', 'Lr', 'Rf', 'Db', 'Sg', 
            'Bh', 'Hs', 'Mt']

def getSymbol(z):
    return ElementList[z-1]

def getZ(ele):
    return ElementList.index(ele) + 1

class testEPDL97(unittest.TestCase):
    def setUp(self):
        """
        import the module
        """
        try:
            from fisx import EPDL97
            self.epdl97 = EPDL97
        except:
            self.edpl97 = None

    def tearDown(self):
        self.epdl97 = None

    def testEPDL97Import(self):
        self.assertTrue(self.epdl97 is not None,
                        'Unsuccessful fisx.EPDL97 import')
    if PYMCA:
        def testEPDL97BindingVersusPyMcaEPDL97Binding(self):
            from fisx import DataDir
            dirname = DataDir.FISX_DATA_DIR
            epdl = self.epdl97(dirname)
            for i in range(1, 99):
                pymca = PyMcaEPDL97.EPDL97_DICT[getSymbol(i)]['binding']
                cpp = epdl.getBindingEnergies(i)
                for key in cpp:
                    difference = abs(cpp[key] - pymca[key])
                    self.assertTrue(difference < 1.0e-7,
                        "Element %s, shell %s , difference %f" %\
                            (getSymbol(i), key, difference))
            epdl = None

        def testEPDL97MuVersusPyMcaEPDL97Mu(self):
            from fisx import DataDir
            dirname = DataDir.FISX_DATA_DIR
            epdl = self.epdl97(dirname)
            x = numpy.linspace(1.0, 80., 157)
            for j in range(1, 100):
                pymca = PyMcaEPDL97.getElementCrossSections(getSymbol(j), x)
                cpp = epdl.getMassAttenuationCoefficients(j, x)
                for key in cpp:
                    for i in range(len(x)):
                        if key == "photoelectric":
                            delta = cpp[key][i] - pymca["photo"][i]
                        else:
                            delta = cpp[key][i] - pymca[key][i]
                        if cpp[key][i] > 0:
                            delta = 100. * (abs(delta)/cpp[key][i])
                            tol = 1.0e-5
                        else:
                            delta = abs(delta)
                            tol = 1.0e-7
                        self.assertTrue(delta < tol,
                            "z = %d, effect = %s, energy = %f, delta = %f" %\
                                (j, key, x[i], delta))
            epdl = None

        def testEPDL97PartialVersusPyMcaPartial(self):
            from PyMca5.PyMca import PyMcaEPDL97
            from fisx import DataDir
            dirname = DataDir.FISX_DATA_DIR
            epdl = self.epdl97(dirname)
            pymca = PyMcaEPDL97.getElementCrossSections('Pb')
            idx = numpy.nonzero((pymca['energy'] > 1.0) & \
                                (pymca['energy'] < 100))[0]
            x = pymca['energy'][idx]
            # test partial photoelectric cross sections
            shellList = ["K", "L1", "L2", "L3", "M1", "M2", "M3", "M4", "M5", "all other"]
            for j in range(1, 100):
                cpp = epdl.getPhotoelectricWeights(j, x)
                for i in range(len(x)):
                    pymca = PyMcaEPDL97.getPhotoelectricWeights(getSymbol(j),
                                                                    shellList,
                                                                    x[i])
                    for ikey in range(len(shellList)):
                        key = shellList[ikey]
                        delta = abs(pymca[ikey]- cpp[key][i])
                        tol = 1.0e-7
                        self.assertTrue(delta < tol,
                            "Default E, z = %d, shell = %s, energy = %f, delta = %f" %\
                                (j, key, x[i], delta))
            print("\nPhotoelectric weights OK at default energies")
            x += 0.123
            for j in range(1, 100):
                cpp = epdl.getPhotoelectricWeights(j, x)
                for i in range(len(x)):
                    if x[i] < 1.0:
                        continue
                    pymca = PyMcaEPDL97.getPhotoelectricWeights(getSymbol(j),
                                                                shellList,
                                                                x[i])
                    for ikey in range(len(shellList)):
                        key = shellList[ikey]
                        delta = abs(pymca[ikey]- cpp[key][i])
                        tol = 1.0e-7
                        self.assertTrue(delta < tol,
                            "z = %d, shell = %s, energy = %f, delta = %f" %\
                                (j, key, x[i], delta))
            print("Photoelectric weights OK at any energy")

def getSuite(auto=True):
    testSuite = unittest.TestSuite()
    if auto:
        testSuite.addTest(\
            unittest.TestLoader().loadTestsFromTestCase(testEPDL97))
    else:
        # use a predefined order
        testSuite.addTest(testEPDL97("testEPDL97Import"))
        if PYMCA:
            testSuite.addTest(testEPDL97("testEPDL97BindingVersusPyMcaEPDL97Binding"))
            testSuite.addTest(\
                testEPDL97("testEPDL97MuVersusPyMcaEPDL97Mu"))
            testSuite.addTest(\
                testEPDL97("testEPDL97PartialVersusPyMcaPartial"))
    return testSuite

def test(auto=False):
    unittest.TextTestRunner(verbosity=2).run(getSuite(auto=auto))

if __name__ == '__main__':
    test()



#/*##########################################################################
#
# The fisx library for X-Ray Fluorescence
#
# Copyright (c) 2014-2016 European Synchrotron Radiation Facility
#
# This file is part of the fisx X-ray developed by V.A. Sole
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
__author__ = "V.A. Sole - ESRF Data Analysis"
import unittest
import sys
import os
import gc
import tempfile

try:
    from PyMca5.PyMca import specfile
    PYMCA = True
except ImportError:
    # do not compare with PyMca because it is not installed
    PYMCA = False

class testSimpleSpecfile(unittest.TestCase):
    def setUp(self):
        """
        import the module
        """
        try:
            from fisx import SimpleSpecfile
            self.specfileClass = SimpleSpecfile
        except:
            self.specfileClass = None
        if self.specfileClass is not None:
            text  = "#F \n"
            text += "\n"
            text += "#S 10  Undefined command 0\n"
            text += "#N 3\n"
            text += "#L First label  Second label  Third label\n"
            text += "10  100  1000\n"
            text += "20  400  8000\n"
            text += "30  900  270000\n"
            text += "\n"
            text += "#S 20  Undefined command 1\n"
            text += "#N 3\n"
            text += "#L First  Second  Third\n"
            text += "1.3  1  1\n"
            text += "2.5  4  8\n"
            text += "3.7  9  27\n"
            text += "\n"
            tmpFile = tempfile.mkstemp(text=False)
            if sys.version < '3.0':
                os.write(tmpFile[0], text)
            else:
                os.write(tmpFile[0], bytes(text, 'utf-8'))
            os.close(tmpFile[0])
            self.fname = tmpFile[1]

            # generate a badly formatted file using \r\n endings
            text = text.replace("\n", "\r\n")
            tmpFile = tempfile.mkstemp(text=False)
            if sys.version < '3.0':
                os.write(tmpFile[0], text)
            else:
                os.write(tmpFile[0], bytes(text, 'utf-8'))
            os.close(tmpFile[0])
            self.fnameCR = tmpFile[1]

    def tearDown(self):
        """clean up any possible files"""
        # make sure the file handle is free
        self._sf = None
        self._scan = None
        # this should free the handle
        gc.collect()
        if self.specfileClass is not None:
            if os.path.exists(self.fname):
                os.remove(self.fname)
            if os.path.exists(self.fnameCR):
                os.remove(self.fnameCR)

    def testSimpleSpecfileImport(self):
        #"""Test successful import"""
        self.assertTrue(self.specfileClass is not None,
                        'Unsuccessful fisx.SimpleSpecfile import')

    def testSimpleSpecfileReading(self):
        #"""Test specfile readout"""
        self.testSimpleSpecfileImport()
        self._sf = self.specfileClass(self.fname)

        # test the number of found scans
        nScans = self._sf.getNumberOfScans()
        self.assertEqual(nScans, 2,
                         'Expected to read 2 scans, read %s' %\
                         nScans)

        # test scan iteration selection method
        labels = self._sf.getScanLabels(1)
        expectedLabels = ['First', 'Second', 'Third']
        self.assertEqual(len(labels), 3,
                         'Expected to read 3 labels, got %d' % len(labels))
        for i in range(3):
            self.assertEqual(labels[i], expectedLabels[i],
                    'Read "%s" instead of "%s"' %\
                     (labels[i], expectedLabels[i]))

        gc.collect()

    def testSimpleSpecfileReadingWithCRLF(self):
        #"""Test specfile readout"""
        self.testSimpleSpecfileImport()
        self._sf = self.specfileClass(self.fnameCR)

        # test the number of found scans
        nScans = self._sf.getNumberOfScans()
        self.assertEqual(nScans, 2,
                         'Expected to read 2 scans, read %s' %\
                         nScans)

        # test scan iteration selection method
        labels = self._sf.getScanLabels(1)
        expectedLabels = ['First', 'Second', 'Third']
        self.assertEqual(len(labels), 3,
                    'Expected to read 3 labels, got %d' % len(labels))
        for i in range(3):
            self.assertEqual(labels[i], expectedLabels[i],
                    'Read <"%s"> instead of "%s" with CRLF endings' %\
                     (labels[i], expectedLabels[i]))

        gc.collect()

    def testSimpleSpecfileReadingCompatibleWithUserLocale(self):
        #"""Test specfile compatible with C locale"""
        self.testSimpleSpecfileImport()
        self._sf = self.specfileClass(self.fname)
        data = self._sf.getScanData(1)
        self._sf = None
        self.assertEqual(data[0] [0], 1.3,
                    'Read %f instead of %f' %\
                    (data[0][0], 1.3))
        self.assertEqual(data[1] [0], 2.5,
                    'Read %f instead of %f' %\
                    (data[1] [0], 2.5))
        self.assertEqual(data[2] [0], 3.7,
                    'Read %f instead of %f' %\
                    (data[2] [0], 3.7))
        gc.collect()

    
    if PYMCA:
        def testSimpleSpecfileVersusPyMca(self):
            import glob
            import PyMca5
            from PyMca5 import PyMcaDataDir
            from PyMca5.PyMcaIO import specfilewrapper as Specfile
            fileList = glob.glob(os.path.join(PyMcaDataDir.PYMCA_DATA_DIR, '*.dat'))
            self.testSimpleSpecfileImport()
            for filename in fileList:
                self._sf = self.specfileClass(filename)
                a = self._sf
                nScans = a.getNumberOfScans()
                sf = Specfile.Specfile(filename)
                for i in range(nScans):
                    scan = sf[i]
                    if PyMca5.version() > "5.1.1":
                        # previous versions of PyMca5 could segfault here
                        labels0 = a.getScanLabels(i)
                        labels1 = scan.alllabels()
                        for j in range(len(labels0)):
                            self.assertEqual(labels0[j], labels1[j],
                                        "Read <%s> instead of <%s>" %\
                                             (labels0[j], labels1[j]))
                    data0 = a.getScanData(i)
                    data1 = scan.data().T
                    M = abs(data0 - data1).max()
                    if M > 1.0e-8:
                        raise ValueError("Error reading data")        

def getSuite(auto=True):
    testSuite = unittest.TestSuite()
    if auto:
        testSuite.addTest(\
            unittest.TestLoader().loadTestsFromTestCase(testSimpleSpecfile))
    else:
        # use a predefined order
        testSuite.addTest(testSimpleSpecfile("testSimpleSpecfileImport"))
        testSuite.addTest(testSimpleSpecfile("testSimpleSpecfileReading"))
        testSuite.addTest(testSimpleSpecfile(\
            "testSimpleSpecfileReadingWithCRLF"))
        testSuite.addTest(\
            testSimpleSpecfile("testSimpleSpecfileReadingCompatibleWithUserLocale"))
        testSuite.addTest(\
            testSimpleSpecfile("testSimpleSpecfileVersusPyMca"))
    return testSuite

def test(auto=False):
    unittest.TextTestRunner(verbosity=2).run(getSuite(auto=auto))

if __name__ == '__main__':
    test()
